-- Curseforge icon made by Freepik from www.flaticon.com

FortunaDiceRoller = LibStub("AceAddon-3.0"):NewAddon("FortunaDiceRoller", "AceConsole-3.0", "AceEvent-3.0")
AceGUI = LibStub("AceGUI-3.0")

VerboseRollString = {}
ConciseRollString = {}
StringIndex = 1

fdr = FortunaDiceRoller

local FirstLightGuilds = {
  ["First Light"] = true;
  ["First Líght"] = true;
}

function fdr:RollDice()

  local TotalDieRoll = 0                --track total so far
  local IxplosionRoll = 0              --imploding/exploding die roll

  -- You must roll at least this well to be effective at all. Otherwise, you fail
  -- and have no effect.
  local difficulty = fdr.db.profile.DifficultyValue

  -- We don't let end users adjust this. It's part of the basic Fortuna rules.
  -- Every (this many) pips rolled gains an extra point of healing/damage.
  local threshold = 5

  local dice = fdr.db.profile.DieCount
  local sides = fdr.db.profile.DiceSides

  local critFail = 1                 --default crit-fail to 1
  local critSuccess = sides          --crit success = highest roll possible

  local DiceModifier = fdr.db.profile.DiceModifier

  if fdr.db.profile.AspectCount > 0 then
    ThisRollOutputString = "Invoked " .. tostring(fdr.db.profile.AspectCount) .. " aspect(s). "
    table.insert(VerboseRollString, ThisRollOutputString)
  end

  if fdr.db.profile.AspectCount > 0 then
    ThisConciseRollString = tostring(fdr.db.profile.AspectCount) .. " Aspects. "
    table.insert(ConciseRollString, ThisConciseRollString)
  end

  for i=1,dice+fdr.db.profile.AspectCount do
    for j=1,dice do
      ThisRoll = math.random(sides)
      TotalDieRoll = TotalDieRoll + ThisRoll

      ThisRollOutputString = "Rolled a d" .. tostring(sides) .. " for " .. tostring(ThisRoll) .. ". "
      table.insert(VerboseRollString, ThisRollOutputString)

      if fdr.db.profile.Ixplosions == true then
        if ThisRoll == critFail or ThisRoll == critSuccess then
          while math.abs(IxplosionRoll) == 0 or math.abs(IxplosionRoll) == critFail or math.abs(IxplosionRoll) == critSuccess do
            IxplosionRoll = math.random(sides)
            if ThisRoll == critFail or IxplosionRoll == critFail then
              IxplosionRoll = -IxplosionRoll
            end
            TotalDieRoll = TotalDieRoll+IxplosionRoll
            ThisRollOutputString = "Rolled an ixplosion d" ..sides .. " for " .. tostring(IxplosionRoll) .. ". "
            table.insert(VerboseRollString, ThisRollOutputString)
            if math.abs(IxplosionRoll) ~= critFail and math.abs(IxplosionRoll) ~= critSuccess then
              break
            end
          end
        end
      end
    end

    ThisVerboseRollString = "Modifier " .. tostring(DiceModifier) .. " added. "
    table.insert(VerboseRollString, ThisVerboseRollString)
    TotalDieRoll = TotalDieRoll + DiceModifier
  end

  ThisConciseRollString = "Rolled " .. dice .. "d" .. sides .. "+" .. DiceModifier .. " for " .. TotalDieRoll
  table.insert(ConciseRollString, ThisConciseRollString)

  ActionEffectiveness = math.floor((TotalDieRoll-difficulty)/threshold)+1

  ThisVerboseRollString = "RESULT: ".. tostring(TotalDieRoll)
  table.insert(VerboseRollString, ThisVerboseRollString)

  if fdr.db.profile.DifficultyCheck == true then
    ThisVerboseRollString = " for " .. tostring(ActionEffectiveness) .. " points against difficulty of " .. tostring(difficulty) .. ". "
    table.insert(VerboseRollString, ThisVerboseRollString)
  end

  if fdr.db.profile.DifficultyCheck == true then
    ThisConciseRollString = " vs. " .. difficulty .. ". (RESULT: " .. ActionEffectiveness .. ")"
    table.insert(ConciseRollString, ThisConciseRollString)
  end

  if fdr.db.profile.OutputTo ~= "CONSOLE" and fdr.db.profile.OutputTo ~= "SAY" then
    ThisVerboseRollString = " <FortunaDiceRoller>"
    table.insert(VerboseRollString, ThisVerboseRollString)
    ThisConciseRollString = " <FDR>"
    table.insert(ConciseRollString, ThisConciseRollString)
  end

  -- We don't need to report ixplosions being disabled if the player is
  -- not in First Light. Other guilds likely won't use this feature.
  -- It's self-evident in verbose mode, so don't do it there.
  if PlayerInFL() then
    if not fdr.db.profile.VerboseMode and not fdr.db.profile.Ixplosions then
      NoIxplosionsString = " (Ixplosions off)"
      table.insert(ConciseRollString, NoIxplosionsString)
    end
  end

  local FinalVerboseRollString = table.concat(VerboseRollString)
  local FinalConciseRollString = table.concat(ConciseRollString)

  if not fdr.db.profile.VerboseMode then
    RollOutput = FinalConciseRollString
  else
    RollOutput = FinalVerboseRollString
  end

  if fdr.db.profile.OutputTo == "SAY" then
    SendChatMessage(RollOutput, fdr.db.profile.OutputTo)
  elseif fdr.db.profile.OutputTo == "PARTY" then
    if UnitInParty("player") then
      SendChatMessage(RollOutput, fdr.db.profile.OutputTo)
    else
      fdr.Print(fdr, "Not in a party. Change output mode and reroll.")
    end
  elseif fdr.db.profile.OutputTo == "RAID" then
    if UnitInRaid("player") then
      SendChatMessage(RollOutput, fdr.db.profile.OutputTo)
    else
      fdr.Print(fdr, "Not in a raid. Change output mode and reroll.")
    end
  elseif fdr.db.profile.OutputTo == "CONSOLE" then
    fdr:Print(RollOutput)
  elseif string.match(fdr.db.profile.OutputTo, "RP") ~= nil and CrossRP ~= nil then
    if CrossRP.relay_on == true then
      SendChatMessage(RollOutput,fdr.db.profile.OutputTo, nil);
    else
      fdr.Print(fdr, "CrossRP installed, but relay seems to be inactive.")
    end
  else
    fdr:Print(fdr, fdr.db.profile.OutputTo)
    fdr:Print(fdr, "Can't figure out what the output mode is. Select one.")
  end

  -- clear tables for next use
  for k in pairs (VerboseRollString) do
    VerboseRollString [k] = nil
  end
  for k in pairs (ConciseRollString) do
    ConciseRollString [k] = nil
  end
end

local defaults = {
  profile = {
    GUIShown = true,
    DieCount = 1,
    DiceSides = 20,
    DiceModifier = 0,
    DifficultyValue = 0,
    DifficultyCheck = false,
    VerboseMode = true,
    OutputTo = "CONSOLE",
    Ixplosions = true,
    AspectCount = 0,
  },
}

function DrawGUI (command)
  if command:lower() == "show" then
    -- WINDOW

    local WindowWidth = 300
    local WindowHeight = 570
    local SliderSizeMultiplier = 0.4

    -- Main window
    local MainFrame = AceGUI:Create("Frame")
    MainFrame:SetCallback("OnClose",function(widget) AceGUI:Release(widget) end)
    MainFrame:SetTitle("Fortuna Dice Roller")
    MainFrame:SetWidth(math.floor(WindowWidth*1.0))
    MainFrame:SetHeight(math.floor(WindowHeight*1.0))
    MainFrame:SetLayout("List")
    MainFrame:EnableResize(false)
    MainFrame:ClearAllPoints()
    MainFrame:SetPoint("CENTER")

    -- GROUPS

    -- Roll area
    local RollFrame = AceGUI:Create("InlineGroup")
    RollFrame:SetTitle("Roll")
    RollFrame:SetFullWidth(true)
    RollFrame:SetLayout("Flow")
    MainFrame:AddChild(RollFrame)

    local DiceOptions = AceGUI:Create("InlineGroup")
    DiceOptions:SetTitle("Dice Options")
    DiceOptions:SetFullWidth(true)
    DiceOptions:SetLayout("Flow")
    MainFrame:AddChild(DiceOptions)

    -- Option area
    local FortunaOptionFrame = AceGUI:Create("InlineGroup")
    FortunaOptionFrame:SetTitle("Behaviour Options")
    FortunaOptionFrame:SetLayout("Flow")
    FortunaOptionFrame:SetFullWidth(true)
    MainFrame:AddChild(FortunaOptionFrame)

    -- Option area
    local OptionFrame = AceGUI:Create("InlineGroup")
    OptionFrame:SetTitle("Fortuna Options")
    OptionFrame:SetLayout("List")
    OptionFrame:SetFullWidth(true)
    MainFrame:AddChild(OptionFrame)

    -- Aspect options
    local AspectFrame = AceGUI:Create("SimpleGroup")
    AspectFrame:SetLayout("Flow")
    AspectFrame:SetHeight(75)
    AspectFrame:SetFullWidth(true)
    OptionFrame:AddChild(AspectFrame)

    -- Chat Output Settings
    local OutputBox = AceGUI:Create("InlineGroup")
    OutputBox:SetTitle("Chat Settings")
    OutputBox:SetLayout("List")
    OutputBox:SetFullWidth(true)
    MainFrame:AddChild(OutputBox)

    -- GROUP CONTENTS

    -- CONTENTS - main options

    -- Die count
    local DieCounter = AceGUI:Create("Slider")
    DieCounter:SetSliderValues(1,10,1)
    DieCounter:SetValue(fdr.db.profile.DieCount)
    DieCounter:SetIsPercent(false)
    DieCounter:SetLabel("Dice")
    DieCounter:SetDisabled(false)
    DieCounter:SetWidth(math.floor(WindowWidth*SliderSizeMultiplier))
    DieCounter:SetCallback("OnMouseUp", function(self, event, value)
      fdr:SetDieCount(value)
    end)
    DiceOptions:AddChild(DieCounter)

    -- Side count
    local SideCounter = AceGUI:Create("Slider")
    SideCounter:SetSliderValues(2,20,2)
    SideCounter:SetValue(fdr.db.profile.DiceSides)
    SideCounter:SetIsPercent(false)
    SideCounter:SetLabel("Sides per die")
    SideCounter:SetDisabled(false)
    SideCounter:SetWidth(math.floor(WindowWidth*SliderSizeMultiplier))
    SideCounter:SetCallback("OnMouseUp", function(self, event, value)
      fdr:SetDiceSides(value)
    end)
    DiceOptions:AddChild(SideCounter)

    -- Modifier slider
    local RollSlider = AceGUI:Create("Slider")
    RollSlider:SetSliderValues(-100,100,1)
    RollSlider:SetValue(fdr.db.profile.DiceModifier)
    RollSlider:SetIsPercent(false)
    RollSlider:SetLabel("Roll modifier")
    RollSlider:SetDisabled(false)
    RollSlider:SetWidth(math.floor(WindowWidth*SliderSizeMultiplier))
    RollSlider:SetCallback("OnMouseUp", function(self, event, value)
      fdr:SetDiceModifier(value)
    end)
    FortunaOptionFrame:AddChild(RollSlider)

    -- Difficulty slider
    local DifficultySlider = AceGUI:Create("Slider")
    DifficultySlider:SetSliderValues(0,100,1)
    DifficultySlider:SetValue(fdr.db.profile.DifficultyValue)
    DifficultySlider:SetIsPercent(false)
    DifficultySlider:SetLabel("Difficulty Check")
    DifficultySlider:SetDisabled(false)
    DifficultySlider:SetWidth(math.floor(WindowWidth*SliderSizeMultiplier))
    DifficultySlider:SetCallback("OnMouseUp", function(self, event, value)
      fdr:SetDifficultyValue(value)
    end)
    FortunaOptionFrame:AddChild(DifficultySlider)

    -- Toggle dice shenanigans
    local DoShenanigansBox = AceGUI:Create("CheckBox")
    DoShenanigansBox:SetType("checkbox")
    DoShenanigansBox:SetValue(fdr.db.profile.Ixplosions)
    DoShenanigansBox:SetLabel("Dice \"ixplode\"")
    DoShenanigansBox:SetTriState(false)
    DoShenanigansBox:SetDisabled(false)
    DoShenanigansBox:SetCallback("OnValueChanged", function(self, event, value)
      fdr:SetIxplosions(value)
    end)
    OptionFrame:AddChild(DoShenanigansBox)

    -- Toggle difficulty check
    local DoDifficultyBox = AceGUI:Create("CheckBox")
    DoDifficultyBox:SetType("checkbox")
    DoDifficultyBox:SetLabel("Do difficulty check")
    DoDifficultyBox:SetTriState(false)
    DoDifficultyBox:SetDisabled(false)
    DoDifficultyBox:SetValue(fdr.db.profile.DifficultyCheck)
    DoDifficultyBox:SetCallback("OnValueChanged", function(self, event, value)
      fdr:SetDifficultyCheck(value)
    end)
    OptionFrame:AddChild(DoDifficultyBox)

    -- CONTENTS - output options

    -- Set up dropdown for output options
    local OutputOptions = {
      ["CONSOLE"] = 'Console',
      ["PARTY"] = 'Party',
      ["RAID"] = 'Raid',
      ["SAY"] = 'Say',
    }

    local OutputTo = AceGUI:Create("Dropdown")
    OutputTo:SetList(OutputOptions)
    if CrossRP ~= nil then
      for i=1,9 do
        OutputTo:AddItem("RP" .. tostring(i), 'RP' .. tostring(i))
      end
    end
    OutputTo:SetValue(fdr.db.profile.OutputTo)
    OutputTo:SetMultiselect(false)
    OutputTo:SetLabel("Output To:")
    OutputTo:SetCallback("OnValueChanged", function(self, event, value)
      fdr:SetOutputTo(value)
    end)
    OutputBox:AddChild(OutputTo)

    local VerboseToggle = AceGUI:Create("CheckBox")
    VerboseToggle:SetType("checkbox")
    VerboseToggle:SetValue(fdr.db.profile.VerboseMode)
    VerboseToggle:SetTriState(false)
    VerboseToggle:SetLabel("Verbose output")
    VerboseToggle:SetCallback("OnValueChanged", function(self, event, value)
      fdr:SetVerboseMode(value)
    end)
    OutputBox:AddChild(VerboseToggle)

    local AspectCount = AceGUI:Create("Slider")
    AspectCount:SetSliderValues(0,10,1)
    AspectCount:SetParent(AspectFrame)
    AspectCount:SetValue(fdr.db.profile.AspectCount)
    AspectCount:SetWidth(150)
    AspectCount:SetLabel("Aspects")
    AspectCount:SetCallback("OnValueChanged", function(self, event, value)
      fdr:SetAspectCount(value)
    end)
    OptionFrame:AddChild(AspectCount)

    -- Roll button

    local RollButton = AceGUI:Create("Button")
    RollButton:SetText("Roll!")
    RollButton:SetCallback("OnClick", function()
      fdr.RollDice(fdr.db.profile.DiceModifier)
    end)
    RollButton:SetFullWidth(true)
    RollFrame:AddChild(RollButton)

  elseif command:lower() == "hide" then
    if MainFrame then
      AceGUI:Release(MainFrame)
    end
  end
end

function PlayerInFL()
  local GuildName = GetGuildInfo("player")
  if FirstLightGuilds[GuildName] == true then
    return true
  end
end

-- Show the GUI if no input is supplied, otherwise handle the chat input.
function fdr:ChatCommand(input)
  if not input or input:trim() == "" then
    -- For now, just show the window. This will be tweaked in later versions.
    DrawGUI("show")
    -- InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
  elseif input:lower() == "show" then
    DrawGUI("show")
    -- elseif input:lower() == "hide" then
    -- DrawGUI("hide")
  else
    LibStub("AceConfigCmd-3.0"):HandleCommand("fdr", "FortunaDiceRoller", input)
  end
end

function fdr:OnInitialize()
  self.db = LibStub("AceDB-3.0"):New("FortunaDiceRollerDB", defaults, true)
  --LibStub("AceConfig-3.0"):RegisterOptionsTable("FortunaDiceRoller", options)
  --self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("FortunaDiceRoller", "FortunaDiceRoller")
  self:RegisterChatCommand("fdr", "ChatCommand")
  self:RegisterChatCommand("fortunadiceroller", "ChatCommand")
  self:RegisterChatCommand("FortunaDiceRoller", "ChatCommand")
end

function fdr:OnEnable(info)
  if fdr.db.profile.GUIShown then DrawGUI("show") end
end

function fdr:GetGUIShown(info)
  return self.db.profile.GUIShown
end

function fdr:SetGUIShown(newValue)
  self.db.profile.GUIShown = newValue
end

function fdr:GetAspectMode(info)
  return self.db.profile.AspectMode
end

function fdr:SetAspectMode(newValue)
  self.db.profile.AspectMode = newValue
end

function fdr:GetAspectCount(info)
  return self.db.profile.AspectCount
end

function fdr:SetAspectCount(newValue)
  self.db.profile.AspectCount = newValue
end

function fdr:GetVerboseMode(info)
  return self.db.profile.VerboseMode
end

function fdr:SetVerboseMode(newValue)
  self.db.profile.VerboseMode = newValue
end

function fdr:GetOutputTo(info)
  return self.db.profile.OutputTo
end

function fdr:SetOutputTo(newValue)
  self.db.profile.OutputTo = newValue
end

function fdr:GetDieCount(info)
  return self.db.profile.DieCount
end

function fdr:SetDieCount(newValue)
  self.db.profile.DieCount = newValue
end

function fdr:GetDiceSides(info)
  return self.db.profile.DiceSides
end

function fdr:SetDiceSides(newValue)
  self.db.profile.DiceSides = newValue
end

function fdr:GetIxplosions(info)
  return self.db.profile.Ixplosions
end

function fdr:SetIxplosions(newValue)
  self.db.profile.Ixplosions = newValue
end

function fdr:GetDiceModifier(info)
  return self.db.profile.DiceModifier
end

function fdr:SetDiceModifier(newValue)
  self.db.profile.DiceModifier = newValue
end

function fdr:GetDifficultyCheck(info)
  return self.db.profile.DifficultyCheck
end

function fdr:SetDifficultyCheck(newValue)
  self.db.profile.DifficultyCheck = newValue
end

function fdr:GetDifficultyValue(info)
  return self.db.profile.DifficultyValue
end

function fdr:SetDifficultyValue(newValue)
  self.db.profile.DifficultyValue = newValue
end
